/* src/admin_chan.c - commandes admin pour gerer les salons
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: admin_chan.c,v 1.59 2003/12/27 16:19:25 romexzf Exp $
 */

#include "main.h"
#include "outils.h"
#include "cs_cmds.h"
#include "fichiers.h"
#include "admin_chan.h"
#include "add_info.h"
#include "del_info.h"
#include "hash.h"
#ifdef SQLLOG
#include "sql_log.h"
#include <mysql.h>
#endif

int admin_chan(aNick *nick, aChan *chan, int parc, char **parv)
{
	char *c = NULL;
	const char *arg = parv[1];
	int i = 0, m = 0;

	if(parc > 1) chan = getchaninfo((c = parv[2]));

	if(!strcasecmp(arg, "suspend"))
	{
		if(parc < 2)
			return csntc(nick, "Syntaxe: %s SUSPEND <salon> [arg]", RealCmd("chan"));

		if(!chan) return csntc(nick, GetReply(nick, L_NOSUCHCHAN), c);

		if(!IsSuspend(chan) && ((parc == 3 && *parv[3] == '%') || parc < 3))
			return csntc(nick, "Veuillez prciser une raison pour suspendre ce salon.");

		if(!IsSuspend(chan))
		{
			time_t timeout = 0;
			if(*parv[3] == '%' && (timeout = convert_duration(++parv[3])) <= 0)
				return csntc(nick, "Dure incorrecte, le format est %%<XjXhXm> X tant le nombre respectif d'units.");

			do_chan_suspend(chan, parv2msg(parc, parv, timeout ? 4 : 3, 250), nick->user->nick, timeout, time(NULL));
			show_suspend(nick, chan);
			cspart(chan, chan->suspend->raison);
			putlog(LOG_CHANS, "SUSPEND ON %s par %s@%s (%s)", c, nick->nick, nick->user->nick, chan->suspend->raison);
		}
		else
		{
			if(getoption("-nolog", parv, parc, 2, -1))
			{
				free(chan->suspend);
				chan->suspend = NULL;
			}
			else chan->suspend->expire = -1;

			csntc(nick, "Le salon \2%s\2 n'est plus suspendu.", c);
			csjoin(chan);
			putlog(LOG_CHANS, "SUSPEND OFF %s par %s@%s", c, nick->nick, nick->user->nick);
		}
		write_achan(chan);
	}
	else if(!strcasecmp(arg, "del"))
	{
		char tmp[400];
		if(parc < 2)
			return csntc(nick, "Syntaxe: %s DEL <salon> <raison>", RealCmd("chan"));

		if(!chan) return csntc(nick, GetReply(nick, L_NOSUCHCHAN), c);

		snprintf(tmp, sizeof tmp, "Salon %s dsenregistr par %s@%s (%s)", c,
			nick->nick, nick->user->nick, (parc > 2) ? parv2msg(parc, parv, 3, 300) : "Aucune raison");

		csntc(nick, "Le salon %s a bien t supprim.", c);
		del_chan(chan, tmp);
		write_chanlist();
	}
	else if(!strcasecmp(arg, "reg"))
	{
		anUser *u;
		if(parc < 3)
			return csntc(nick, "Syntaxe: %s REG <salon> <Username|%%nick> [description]", RealCmd("chan"));

		if(chan) return csntc(nick, GetReply(nick, L_ALREADYREG), c);

		if(!chan_check(c, nick)) return 0;

		if(!(u = ParseNickOrUser(nick, parv[3]))) return 0;

		chan = add_chan(c, (parc >= 4) ? parv2msg(parc, parv, 4, 300) : "Adminreg");
		add_access(u, c, OWNERLEVEL, A_OP | A_PROTECT, time(NULL));
		csjoin(chan);
		append_file(CHANS_FILE,c);
		write_achan(chan);
		write_anuser(u);
		csntc(nick, "%s a t enregistr pour %s.", c, u->nick);
	}
	else if(!strcasecmp(arg, "list"))
	{
		int description = getoption("-description", parv, parc, 2, 0);
		int modes = getoption("-modes", parv, parc, 2, 0), users = getoption("-users", parv, parc, 2, 1);
		int defmodes = getoption("-defmodes", parv, parc, 2, 0);
		int url = getoption("-url", parv, parc, 2, 0), count = getoption("-count", parv, parc, 2, 1);
		int topic = getoption("-topic", parv, parc, 2, 0), j = 0;

		if(parc < 2 || *parv[2] == '-') m = 1;
		if(modes) modes = cmodetoflag(0, parv[modes]);
		if(defmodes) defmodes = cmodetoflag(0, parv[defmodes]);

		for(;j < CHANHASHSIZE;j++)
			for(chan = chan_tab[j];chan;chan = chan->next)
				if((m || !match(parv[2], chan->chan))
				&& (!description || !match(parv[description], chan->description))
				&& (!modes || (chan->mode.modes & modes) == modes)
				&& (!defmodes || (chan->defmodes.modes & defmodes) == defmodes)
				&& (!url || !match(parv[url], chan->url))
				&& (!topic || !match(parv[topic], chan->topic))
				&& chan->users >= users && (++i <= count || !count))
				{
					if(i == 1) csntc(nick, "\2Liste des salons enregistrs :");
					if(!IsSuspend(chan)) csntc(nick, "  %s (%s) [%d]", chan->chan, GetCModes(chan->mode), chan->users);
					else csntc(nick, "  %s (%s) (\2Suspendu\2)", chan->chan, GetCModes(chan->defmodes));
				}

		csntc(nick, "Il y a un total de\2 %d\2 salon%s enregistr%s correspondants.", i, PLUR(i), PLUR(i));
	}
	else if(!strcasecmp(arg, "suspendlist"))
	{
		int from = getoption("-from", parv, parc, 2, 0), j = 0;
		if(parc < 2 || *parv[2] == '-') m = 1;

		for(;j < CHANHASHSIZE;j++)
			for(chan = chan_tab[j];chan;chan = chan->next)
				if(IsSuspend(chan)
					&& (m || !match(parv[2], chan->chan))
					&& (!from || !strcasecmp(parv[from], chan->suspend->from)))
				{
					if(!i++) csntc(nick, "\2Liste des salons suspendus :");
					csntc(nick, "  %s Par %s Expire %s Raison: %s",
							chan->chan, chan->suspend->from,
							chan->suspend->expire ? get_time(chan->suspend->debut + chan->suspend->expire) : "Jamais",
							chan->suspend->raison);
				}

		csntc(nick, "Il y a un total de\2 %d\2 salons suspendus correspondant  vos critres.", i);
	}
	else if(!strcasecmp(arg, "join"))
	{
		if(parc < 2) return csntc(nick, "Syntaxe: %s JOIN <salon> [-force]", RealCmd("chan"));

		if(!chan) return csntc(nick, GetReply(nick, L_NOSUCHCHAN), c);

		if(IsSuspend(chan))
			return csntc(nick, "%s est actuellement suspendu pour %s, utilisez '%s SUSPEND %s' pour lever le suspend.",
				c, chan->suspend->raison, RealCmd("chan"), c);

		if(CJoined(chan) && !getoption("-force", parv, parc, 2, -1))
			return csntc(nick, "Je suis dj sur %s (En cas de desynch, utilisez l'argument '-force')", c);

		csjoin(chan); /* nicklist restaure dans le csjoin */
	}
	else if(!strcasecmp(arg, "part"))
	{
		if(parc < 2) return csntc(nick, "Syntaxe: %s PART <salon> [raison]", RealCmd("chan"));

		if(!chan) return csntc(nick, GetReply(nick, L_NOSUCHCHAN), c);

		cspart(chan, (parc > 2) ? parv2msg(parc, parv, 3, 300) : "Aucune Raison");
	}
	else if(!strcasecmp(arg, "setowner"))
	{/* chan setowner # gna [leve]*/
		anUser *newu;
		aLink *lp;
		anAccess *old = NULL, *new;
		int newlevel = 0;

		if(parc <3) return csntc(nick, "Syntaxe: %s SETOWNER <chan> <nouvel owner> [nouveau level de l'ancien owner]", RealCmd("chan"));
		if(parc > 3 && is_num(parv[4])) newlevel = atoi(parv[4]);
		if(newlevel <= 0 || newlevel >= OWNERLEVEL)
			return csntc(nick, GetReply(nick, L_VALIDLEVEL));

		if(!chan) return csntc(nick, GetReply(nick, L_NOSUCHCHAN), c);
		if(!(newu = getuserinfo(parv[3]))) return csntc(nick, GetReply(nick, L_NOSUCHUSER), parv[3]);

		for(lp = chan->access;lp && lp->value.a->level != OWNERLEVEL;lp = lp->next);/* recherche de l'owner*/
		if(lp) old = lp->value.a;

		if((new = GetAccessIbyUserI(newu, chan)) && new->level == OWNERLEVEL)/* le futur owner l'est dj*/
			return csntc(nick, "%s est dj l'owner de %s.", newu->nick, c);

		if(new) new->level = OWNERLEVEL;
		else add_access(newu, c, OWNERLEVEL, (A_OP | A_PROTECT), time(NULL));

		if(old)
		{
			if(newlevel) old->level = newlevel;
			else del_access(old->user, chan);
			write_anuser(old->user);
		}
		write_anuser(newu);
		csntc(nick, "%s est le nouvel owner de %s.", newu->nick, c);
	}
#ifdef SQLLOG
	else if(!strcasecmp(arg, "historique"))
	{
		time_t date = 0;
		MYSQL_RES *result;
		MYSQL_ROW row;
		int cmd, user;
		char tmp[150] = "SELECT * FROM chancmdslog WHERE chan='%s' AND TS>=%T";

		if(parc < 2) return csntc(nick, "Syntaxe: %s HISTORIQUE <salon> [%%XjXh] [-cmd <cmd>] [-user <user>]", RealCmd("chan"));
		if(!chan) return csntc(nick, GetReply(nick, L_NOSUCHCHAN), c);
		if(parc >= 3 && *parv[3] == '%' && (date = convert_duration(++parv[3])) <= 0)
			return csntc(nick, "Dure incorrecte, le format est %%<XjXhXm> X tant le nombre respectif d'units.");

		if((cmd = getoption("-cmd", parv, parc, 2, 0))) strcat(tmp, " AND cmd='%s'");
		if((user = getoption("-user", parv, parc, 2, 0))) strcat(tmp, " AND user='%s'");

		if(sql_query(tmp, chan->chan, date ? time(NULL) - date : 0,
			cmd ? parv[cmd] : (user ? parv[user] : ""), user ? parv[user] : "") < 0) return csntc(nick, "La requte SQL a chou.");

		if(!(result = mysql_store_result(bot.sql_id)) || !(row = mysql_fetch_row(result)))
			return csntc(nick, "Aucune entre trouve.");

		if(date) csntc(nick, "Commandes effectues sur \2%s\2 (Par %s) depuis %s.", chan->chan, user ? parv[user] : "tout le monde", duration(date));
		else csntc(nick, "Commandes effectues sur \2%s\2 (Par %s).", chan->chan, user ? parv[user] : "tout le monde");

		for(;row;row = mysql_fetch_row(result)) csntc(nick, "%s: \2%s\2 [%s] par \2%s\2", get_time(strtoul(row[2], NULL, 10)), row[3], NONE(row[4]), row[0]);
		mysql_free_result(result);
	}
#endif
	else if(!strcasecmp(arg, "chansetct"))
	{
		if(parc < 3) return csntc(nick, "Syntaxe: %s chansetct <salon> <ct>", RealCmd("chan"));
		if(!chan) return csntc(nick, GetReply(nick, L_NOSUCHCHAN), c);
		chan->creation_time = strtoul(parv[3], NULL, 10);
		csntc(nick, "Creation time de %s mis au %s", c, get_time(chan->creation_time));
		write_achan(chan);
	}
	else return csntc(nick, GetReply(nick, L_UNKNOWNOPTION), arg);
	return 1;
}

int whoison(aNick *nick, aChan *chan, int parc, char **parv)
{
	int first = 1, size = 0;
	char buf[451] = {0}, tmp[NICKLEN + 16];
	aLink *lp = chan->members;

	if(!lp) return csntc(nick, "Aucun User sur %s.", chan->chan);
	for(;lp;lp = lp->next)
	{
		int tmps = size, p = sprintf(tmp, " %s%s", GetChanPrefix(lp->value.j->nick, lp->value.j, 1), lp->value.j->nick->nick);
		size += p;
		if(size >= 450)
		{
			if(first) csntc(nick, "Un total de %d Users sur %s:%s...", chan->users, chan->chan, buf);
			else csntc(nick, "              :%s...", buf);
			first = 0;
			strcpy(buf, tmp);
			size = p;
		}
		else strcpy(&buf[tmps], tmp);
	}
	if(first && *buf) csntc(nick, "Un total de %d Users sur %s:\2%s", chan->users, chan->chan, buf);
	else if(*buf) csntc(nick, "              :%s", buf);

	return 1;
}

void do_chan_suspend(aChan *chan, const char *raison, const char *nick, time_t expire, time_t debut)
{
	if(!chan->suspend) chan->suspend = calloc(1, sizeof(struct suspendinfo));
	Strncpy(chan->suspend->raison, raison, 250);
	Strncpy(chan->suspend->from, nick, NICKLEN);
	chan->suspend->expire = expire;
	chan->suspend->debut = debut;
	return;
}

void show_suspend(aNick *nick, aChan *chan)
{
	if(!chan->suspend) return; /* on aurait jamais du l'envoyer l*/
	if(IsSuspend(chan))
	{
		char buf[200];
		Strncpy(buf, get_time(chan->suspend->debut), sizeof buf -1);
		csntc(nick, "Le salon %s \2est\2 suspendu par %s (le %s). Expire %s",
			chan->chan, chan->suspend->from, buf,
			chan->suspend->expire ? get_time(chan->suspend->debut + chan->suspend->expire) : "Jamais");
	}
	else csntc(nick, "Le salon tait suspend par %s", chan->suspend->from);
	csntc(nick, "Raison: %s", chan->suspend->raison);
	return;
}
