/* src/chanserv.c - Diverses commandes sur le module chanserv
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Gay@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: chanserv.c,v 1.70 2003/12/27 23:36:17 romexzf Exp $
 */

#include "main.h"
#include "hash.h"
#include "cs_cmds.h"
#include "outils.h"
#include "chanserv.h"
#include "ban.h"
#include "add_info.h"
#include "del_info.h"
#include "fichiers.h"

/*
 * show_access <chan> <recherche>
 */
int show_access(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	register aLink *lp = chaninfo->access;
	char *nick2 = parv[2], comp = '\0';
	int i = 0, autoop, protect, suspend, autovoice, l = 0, wild = 0;

	if((chaninfo->mode.modes & (C_MSECRET | C_MPRIVATE)) /* salon +s ou +p */
		&& (!nick->user || !GetAccessIbyUserI(nick->user, chaninfo))	/* non logu ou n'a pas d'accs */
		&& !IsAnAdmin(nick->user) && !getjoininfo(nick, chaninfo->chan)) /* ET ni admin, ni prsent */
			return csntc(nick, GetReply(nick, L_YOUNOACCESSON), chaninfo->chan);

	if(*nick2 == '>' || *nick2 == '<' || *nick2 == '=')
	{
		comp = *nick2++;
		if(!*nick2 || !is_num(nick2))
			return csntc(nick, "La valeur du niveau doit tre numrique derrire <,> ou =");
		l = atoi(nick2);
	}
	else wild = (strchr(nick2, '*') || strchr(nick2, '?'));

	autoop = getoption("-autoop", parv, parc, 2, -1);
	autovoice = getoption("-autovoice", parv, parc, 2, -1);
	protect = getoption("-protect", parv, parc, 2, -1);
	suspend = getoption("-suspend", parv, parc, 2, -1);
	csntc(nick, GetReply(nick, L_ACCESSLIST), chaninfo->chan);

	for(;lp;lp = lp->next)
	{
		if(!AWait(lp->value.a)	&& (
			 (comp != '\0' && ((comp == '>' && lp->value.a->level > l) /* recherche */
			                 || (comp == '<' && lp->value.a->level < l)	/* via level */
			                 || (comp == '=' && l == lp->value.a->level)))
			  || (*nick2 == '*' && !*(nick2+1)) /* recherche sur '*' */
			  || (wild /* ou sur un vrai mask */
			  		&& !match(nick2, lp->value.a->user->nick))
			  || !strcasecmp(nick2, lp->value.a->user->nick)) /* ou access exact */
			&& (!autoop || AOp(lp->value.a)) /* ET match des options Aop/Avoice/Suspend etc */
			&& (!autovoice || AVoice(lp->value.a))
			&& (!protect || AProtect(lp->value.a))
			&& (!suspend || ASuspend(lp->value.a)))
		{
			show_accessn(lp->value.a, lp->value.a->user, nick);
			csntc(nick, "-");
			i++;
		}
	}
	return csntc(nick, GetReply(nick, L_TOTALFOUND), i, PLUR(i), PLUR(i));
}

void show_accessn(anAccess *acces, anUser *user, aNick *num)
{
	csntc(num, GetReply(num, L_ACUSER), user->nick, acces->level, user->n ? "Oui" : "Non");
	csntc(num, "\2Options:\2%s", GetAccessOptions(acces));
	if(acces->lastseen == 1) csntc(num, GetReply(num, L_ACONCHAN));
	else csntc(num, GetReply(num, L_ACLASTSEEN), get_time(acces->lastseen));
	if(acces->info) csntc(num, "\2Infoline:\2 %s", acces->info);
	return;
}

/*
 * add_user <salon> <user> <level>
 */
int add_user(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	anUser *u;
	anAccess *a;
	char *username = parv[2], *salon = parv[1];
#ifdef USE_MEMOSERV
	char memo[MEMOLEN + 1];
#endif
	int niveau;

	if(!(u = ParseNickOrUser(nick, username))) return 0;

	if(!is_num(parv[3]) || (niveau = atoi(parv[3])) < 1 || niveau >= OWNERLEVEL)
		return csntc(nick, GetReply(nick, L_VALIDLEVEL));

	if(niveau >= ChanLevelbyUserI(nick->user, chaninfo) && !IsAnAdmin(nick->user))
		return csntc(nick, GetReply(nick, L_MAXLEVELISYOURS));

	if(u->flag & U_PREJECT)
		return csntc(nick, GetReply(nick, L_XREFUSEACCESS), u->nick);

	for(a = u->accesshead;a && a->c != chaninfo;a = a->next);
	if(a)
	{
		if(AWait(a)) csntc(nick, GetReply(nick, L_ALREADYAWAIT), u->nick);
		else csntc(nick, GetReply(nick, L_ALREADYACCESS), u->nick, salon);
		return 0;
	}

	if(u->flag & U_PACCEPT)
	{
		add_access(u, salon, niveau, (niveau >= 400) ? (A_OP | A_PROTECT) : 0, time(NULL));
		csntc(nick, GetReply(nick, L_XHASACCESS), u->nick, niveau, salon);
		if(u->n) csntc(u->n, GetReply(nick, L_YOUHAVEACCESS), nick->nick, niveau, salon);
#ifdef USE_MEMOSERV
		else
		{
			snprintf(memo, MEMOLEN, GetReply(nick, L_YOUHAVEACCESS), nick->nick, niveau, salon);
			add_memo(u, nick->user->nick, time(NULL), memo, 0);
		}
#endif
	}
	else
	{
		add_access(u, salon, niveau, (niveau >= 400) ? (A_OP | A_PROTECT | A_WAITACCESS) : A_WAITACCESS, time(NULL));
		csntc(nick, GetReply(nick, L_XHASAWAIT), u->nick, salon);

		if(u->n)
		{
			csntc(u->n, GetReply(nick, L_YOUHAVEAWAIT1), nick->nick, niveau, salon);
			csntc(u->n, GetReply(nick, L_YOUHAVEAWAIT2), cs.nick, RealCmd("myaccess"),
				salon, cs.nick, RealCmd("myaccess"), salon);
		}
#ifdef USE_MEMOSERV
		else
		{
			snprintf(memo, MEMOLEN, GetReply(nick, L_YOUHAVEAWAIT1), nick->nick, niveau, salon);
			add_memo(u, nick->user->nick, time(NULL), memo, 0);
		}
#endif
	}
	write_anuser(u);
	return 1; /* on sait jamais :P */
}

/*
 * del_user <salon> <user>
 */
int del_user(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	anUser *u;
	anAccess *a;
	const char *salon = parv[1];

	if(!(u = ParseNickOrUser(nick, parv[2]))) return 0;

	if(!(a = GetAccessIbyUserI(u, chaninfo)))
		return csntc(nick, GetReply(nick, L_XNOACCESSON), u->nick, salon);

	if(ChanLevelbyUserI(nick->user, chaninfo) <= a->level && !IsAnAdmin(nick->user))
		return csntc(nick, GetReply(nick, L_GREATERLEVEL), u->nick);

	if(a->level == OWNERLEVEL) return csntc(nick, GetReply(nick, L_XISOWNER), u->nick);

	del_access(u, chaninfo);
	write_anuser(u);

	csntc(nick, GetReply(nick, L_OKDELETED), u->nick);
	return 1;
}

int kick(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	const char *chan = parv[1], *cible = parv[2];
	aNick *targ;

	if(!strchr(cible, '*') && !strchr(cible, '?'))
	{	/*la cible est un nick unique .. */

		if(!strcasecmp(cs.nick, cible))
			return csntc(nick, "Non!");

		if(!(targ = GetMemberIbyNick(chaninfo, cible)))
			return csntc(nick, GetReply(nick, L_NOTONCHAN), cible, chaninfo->chan);

		if(!check_protect(nick, targ, chaninfo)) return 0;

		cskick(chan, targ->numeric, "(\2%s\2) %s", nick->user->nick,
			(parc < 3) ? DEFRAISON : parv2msg(parc, parv, 3, 150));
	}
	else
	{	/* kick sur un mask*/
		const char *r = (parc < 3) ? DEFRAISON : parv2msg(parc, parv, 3, 150);
		aLink *lp = chaninfo->members;

		for(;lp;lp = lp->next)
		{
			targ = lp->value.j->nick;

			if(!match(cible, GetNUHbyNick(targ, 0)) && nick != targ && check_protect(nick, targ, chaninfo) && !(targ->flag & N_SERVICE))
				cskick(chan, targ->numeric, "(\2%s\2) %s", nick->user->nick, r);
		}
	}
	return 1;
}


int verify(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	aNick *n;

	if(!strcasecmp(parv[1], cs.nick)) return csntc(nick, "Yeah, c'est bien moi :)");
	if(!(n = getnickbynick(parv[1]))) return csntc(nick, GetReply(nick, L_NOSUCHNICK), parv[1]);
	if(!n->user) return csntc(nick, GetReply(nick, L_NOTLOGUED), n->nick);

	if(IsAnAdmin(nick->user) || n == nick)
		csntc(nick, "%s!%s@%s est logu sous l'username %s %s %s", n->nick, n->ident, n->host,
			n->user->nick, IsAnAdmin(n->user) ? "- Administrateur des Services" : "", IsOper(n) ? "- IRCop" : "");
	else csntc(nick, "%s est logu sous l'username %s %s %s", n->nick, n->user->nick,
			IsAnAdmin(n->user) ? "- Administrateur des Services" : "", IsOper(n) ? " - IRCop" : "");
	return 1;
}

int mode(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	int a = 0;
	char *modes = parv[2], *c = strchr(modes, 'o');

	if(c || (c = strchr(modes, 'b')) || (c = strchr(modes, 'v'))
	 || (!IsOper(nick) && (c = strchr(modes, 'O'))))
		return csntc(nick, GetReply(nick, L_CANTCHANGEMODE), *c);

	if(chaninfo->cml && !IsAnAdmin(nick->user) && ChanLevelbyUserI(nick->user, chaninfo) < chaninfo->cml)
		return csntc(nick, GetReply(nick, L_CMLIS), chaninfo->cml);

	if(strchr(modes, 'k')) a++;
	if(strchr(modes, 'l')) a++;

	csmode(chaninfo, 0, "%s %s %s", modes, (a && parc > 2) ? parv[3] : "", (a > 1 && parc > 3) ? parv[4] : "");
	return 1;
}

int topic(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	if(CLockTopic(chaninfo)) return csntc(nick, GetReply(nick, L_LOCKTOPIC), parv[1]);

	cstopic(chaninfo, parv2msg(parc, parv, 2, TOPICLEN));
	write_achan(chaninfo);
	return 1;
}

int rdeftopic(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	if(CLockTopic(chaninfo)) return csntc(nick, GetReply(nick, L_LOCKTOPIC), parv[1]);

	if(*chaninfo->deftopic) cstopic(chaninfo, chaninfo->deftopic);
	return 1;
}

int rdefmodes(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	csmode(chaninfo, 0, "-%s", GetCModes(chaninfo->mode));
	if(chaninfo->defmodes.modes) csmode(chaninfo, 0, "+%s", GetCModes(chaninfo->defmodes));
	return 1;
}

int info(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	anAccess *acces = GetAccessIbyUserI(nick->user, chaninfo);

	if(!acces) return 0;

	if(parc < 2)
		return csntc(nick, GetReply(nick, L_INFOLINEORNONE));

	if(!strcasecmp(parv[2], "none"))
	{
		if(acces->info) free(acces->info), acces->info = NULL;
		csntc(nick, GetReply(nick, L_OKDELETED), chaninfo->chan);
	}
	else
	{
		str_dup(&acces->info, parv2msg(parc, parv, 2, 199));
		csntc(nick, GetReply(nick, L_INFOCHANGED), chaninfo->chan, acces->info);
	}
	write_anuser(nick->user);
	return 1;
}

int invite(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	aNick *inv = nick;
	if(parc > 1 && strcasecmp(parv[2], nick->nick))
	{
		if(!IsAnAdmin(nick->user)) return csntc(nick, GetReply(nick, L_CANTINVELSE));
		else if(!(inv = getnickbynick(parv[2]))) return csntc(nick, GetReply(nick, L_NOSUCHNICK), parv[2]);
	}

	if(getjoininfo(inv, chaninfo->chan))
		return csntc(nick, GetReply(nick, L_ALREADYONCHAN), inv->nick, chaninfo->chan);

	if(is_ban(inv, chaninfo))
		return csntc(nick, GetReply(nick, L_XISBANCANTINV), inv->nick, chaninfo->chan);

	putserv("%s " TOKEN_INVITE " %s :%s", cs.num, inv->nick, chaninfo->chan);
	putserv("%s " TOKEN_WALLCHOPS " %s :Invitation de %s",	cs.num,	chaninfo->chan, inv->nick);
	return 1;
}

int clearmodes(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	csmode(chaninfo, 0, "-%s", GetCModes(chaninfo->mode));
	return 1;
}

/*
 * see_alist #
 */
int see_alist(aNick *nick, aChan *chan, int parc, char **parv)
{
	aLink *lp = chan->access;
	int i = 0;

	for(;lp;lp = lp->next)
		if(lp->value.a->user->n)
		{
			if(!i)
			{
				csntc(nick, "Liste des identifis sur %s", chan->chan);
				csntc(nick, "\2Pseudo Actuel Username Niveau");
			}
			csntc(nick, "%-13s %-8s %-6d", lp->value.a->user->n->nick, lp->value.a->user->nick, lp->value.a->level);
			++i;
		}
	if(!i) csntc(nick, GetReply(nick, L_NOINFOAVAILABLE));
	return 1;
}

int admin_say(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	putserv("%s " TOKEN_PRIVMSG " %s :%s", cs.num, parv[1], parv2msg(parc, parv, 2, 400));
	return 1;
}

int admin_do(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	putserv("%s " TOKEN_PRIVMSG " %s :\1ACTION %s\1", cs.num, parv[1], parv2msg(parc, parv, 2, 400));
	return 1;
}

