/* src/del_info.c - Suppression d'informations en mmoire
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: del_info.c,v 1.53 2003/12/27 16:19:26 romexzf Exp $
 */

#include "main.h"
#include "outils.h"
#include "del_info.h"

int del_abadchan(const char *chaine)
{
	struct badchans *current = badchead, *last = NULL;

	for(;current; last = current, current = current->next)
		if(!strcasecmp(chaine, current->chaine))
		{
			if(last) last->next = current->next;
			else badchead = current->next;
			free(current);
			return 1;
		}
	return 0;
}

void del_access(anUser *user, aChan *chan)
{
	anAccess *acces = user->accesshead, *last = NULL;
	aLink *lp = NULL, *lpp = NULL;

	for(;acces;last = acces, acces = acces->next)
		if(acces->c == chan)
		{
			if(last) last->next = acces->next;
			else user->accesshead = acces->next;
			break;
		}

	for(lp = chan->access;lp;lpp = lp, lp = lp->next)
		if(lp->value.a->user == user)
		{
			if(lpp) lpp->next = lp->next;
			else chan->access = lp->next;

			if(lp->value.a->info) free(lp->value.a->info);
			free(lp->value.a);
			free(lp); /* ne pas oublier de free() le link, -memory leak-*/
			return;
		}
	return;
}

void del_killinfo(aNick *nick, const char *against)
{
   aKill *current, *previous = NULL;

   current = killhead;
   while(current)/* del de la queue des kills si c'est celui qui pointe vers le nick */
   {			/*  et que nick->nick correspond au 'against', si NULL tout kill est retir*/
      if(current->nick == nick && (!against || !strcasecmp(nick->nick, against)))
      {
		  if(current == killhead) killhead = killhead->next;
		  else previous->next = current->next;
		  free(current);
		  return;
      }
      else
      {
         previous = current;
         current = current->next;
      }
   }
}

int del_abadnick(const char *chaine)
{
	struct badnicks *bn = badnhead, *previous = NULL;

	for(;bn && strcasecmp(chaine, bn->chaine);previous = bn, bn = bn->next);
	if(bn)
	{
		if(previous) previous->next = bn->next;
		else badnhead = bn->next;
		free(bn);
		return 1;
	}
	return 0;
}

void del_join(aNick *nick, const char *chan)
{
	aJoin *current, *previous = NULL;

	for(current = nick->joinhead;current;previous = current, current = current->next)
		if(!strcasecmp(chan, current->channel))
		{
			if(previous) previous->next = current->next;
			else nick->joinhead = current->next;

			if(nick->user && current->chan) /* on fout le lastseen comme il faut */
			{
				anAccess *acces = GetAccessIbyUserI(nick->user, current->chan);
				if(acces) acces->lastseen = time(NULL);
			}
			if(current->chan) del_link(current->chan, nick); /* on del le link*/
			free(current);
			break;
		}

	return;
}

void del_link(aChan *chan, aNick *nick)
{
	register aLink *l = chan->members, *lp = NULL;

	chan->users--;
	for(;l;lp = l, l = l->next)
		if(l->value.j->nick == nick)
		{
			if(lp) lp->next = l->next;
			else chan->members = l->next;
			free(l);
			return;
		}
}

void del_ban(aChan *chan, aBan *ban)
{
	if(chan->banhead == ban)
	{
		chan->banhead = ban->next;
		if(ban->next) chan->banhead->last = NULL;
	}
	else
	{
		ban->last->next = ban->next;
		if(ban->next) ban->next->last = ban->last;
	}
	free(ban->raison);
	free(ban);
	return;
}

void del_ignore(const char *host)
{
	struct ignore *i = ignorehead, *previous = NULL;

	for(;i;previous = i, i = i->next)
		if(!strcasecmp(host, i->host))
		{
			if(previous) previous->next = i->next;
			else ignorehead = i->next;
			free(i);
			return;
		}
}
