/* src/vote.c - Effectuer un vote
 * Copyright (C) 2002-2003 Inter System (Progs@Inter-System.Net)
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: vote.c,v 1.16 2003/12/27 16:19:27 romexzf Exp $
 */

#include "main.h"
#ifdef HAVE_VOTE
#include "vote.h"
#include "cs_cmds.h"
#include "fichiers.h"
#include "hash.h"
#include "outils.h"

struct votes vote[20];
char vote_topic[MEMOLEN + 1];
int vote_actif, vote_nbprop, intbuf[20];

static int *class_votes(int prout[])
{
	int i = 0, isgood = 0;
	for(;i<= vote_nbprop;i++) intbuf[i] = i;
	while(!isgood)
	{
		isgood = 1;
		for(i = 0;i < vote_nbprop;i++)
		{
			if(prout[i] < prout[i+1])
			{
				int j = prout[i+1];
				prout[i+1] = prout[i];
				prout[i] = j;
				j = intbuf[i+1];
				intbuf[i+1] = intbuf[i];
				intbuf[i] = j;
				isgood = 0;
			}
		}
	}
	return intbuf;
}

void show_vote(aNick *nick)
{
	unsigned int i = 0;
	csntc(nick, "\37\2Sujet\37:\2 %s", vote_topic);
	csntc(nick, "\37\2Propositions\37:\2");
	for(;strcmp(vote[i].prop, "end");i++) csntc(nick, "%d. %s", i+1, vote[i].prop);
	return;
}
/*
 * vote_results
 */
int vote_results(aNick *nick, aChan *c, int parc, char **parv)
{
	float f = 0;
	int i = 0, prout[20], *sex;

	if(vote_actif)
		return csntc(nick, "Vous ne pouvez pas voir les rsultats, le vote est encore en cours.");

	if(!vote_nbprop) return csntc(nick, "Il n'y a aucun vote existant.");

	csntc(nick, "\2Sujet:\2 %s", vote_topic);
	csntc(nick, "Rsultats:");
	for(;strcmp(vote[i].prop, "end");i++)
	{
		f += vote[i].voies;
		prout[i] = vote[i].voies;
	}
	sex = class_votes(prout);
	for(i = 0;i <= vote_nbprop;i++)
		csntc(nick, "%d. %3.2f%% [%d] \2%s", i+1, (vote[sex[i]].voies/f) * 100, vote[sex[i]].voies, vote[sex[i]].prop);
	return 1;
}

/*
 * voter parv[1] = n de la proposition
 */
int voter(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	int nb_prop = 0;

	if(!vote_actif)
	{
		if(!vote_nbprop) return csntc(nick, "Le vote n'est pas actif");
		else
			return csntc(nick, "Le vote est clos. Pour voir les rsultats, tapez /%s %s", cs.nick, RealCmd("results"));
	}

	if(!parc)
	{
		csntc(nick, "Un vote est actuellement en cours.");
		show_vote(nick);
		if(nick->user && !nick->user->vote) csntc(nick, "Pour votez utilisez: \2/%s %s <n de la proposition>\2", cs.nick, RealCmd("voter"));
		return 1;
	}

	if(nick->user->vote == -1)
		return csntc(nick, "Vous avez enregistr votre username aprs l'ouverture du vote. Vous ne pourrez voter qu'au prochain vote");

	if(nick->user->vote) return csntc(nick, "Vous avez dj vot.");

	if(!is_num(parv[1])) return csntc(nick, "Le n de la proposition doit tre un nombre.");

	nb_prop = atoi(parv[1]);

	if(nb_prop < 1 || --nb_prop > vote_nbprop)
		return csntc(nick, "Les propositions vont de 1  %d", 1 + vote_nbprop);

	nick->user->vote = (nb_prop + 1);
	vote[nb_prop].voies++;
	csntc(nick, "Votre vote pour \2%s\2 a bien t pris en compte.", vote[nb_prop].prop);
	write_votes();
	write_anuser(nick->user);
	return 1;
}

/*
 * vote
 *
 * - parv[1] = "NEW" :
 *     parv[2] = nb de choix
 *     parv[3] = :sujet du vote
 * - parv[1] = "PROPOSITION" :
 *     parv[2] = :nom de la proposition
 * - parv[1] = "CLORE" : (arrete le vote)
 * - parv[1] = "RESULTS" : (voir les rsultats)
 */
int do_vote(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	char *cmd;

	if(!parc)
	{
		csntc(nick, "Le vote est \2%sACTIF\2", vote_actif ? "" : "IN");
		if(vote_actif && vote_nbprop) show_vote(nick);
		return 1;
	}

	cmd = parv[1];

	if(!strcasecmp(cmd, "NEW"))
	{
		anUser *u;
		char *msg = NULL;
		int i = 0;

		if(parc < 3)
			return csntc(nick, "Syntaxe: %s NEW <nombre de propositions> :<sujet>", RealCmd("vote"));

		if(vote_actif) return csntc(nick, "Le vote est dj actif");

		if(!is_num(parv[2]))
			return csntc(nick, "Le nombre de proposition doit tre une valeur numrique");

		if((vote_nbprop = atoi(parv[2])) < 2 || vote_nbprop > 20)
			return csntc(nick, "Il peut y avoir de 2  20 propositions possibles.");

		msg = parv2msg(parc, parv, 3, MEMOLEN + 3);
		if(*msg==':') msg++;

		if(strlen(msg) > MEMOLEN)
			return csntc(nick, "La longueur du sujet du vote est limite  %d caractres", MEMOLEN);

		Strncpy(vote_topic, msg, MEMOLEN);

		for(;i < USERHASHSIZE;i++) for(u = user_tab[i];u;u = u->next) u->vote = 0;
		csntc(nick, "Le vote est en cours de cration.");
		csntc(nick, "\2Sujet:\2 %s", vote_topic);
		csntc(nick, "Vous devez maintenant dfinir les propositions avec: "
				"\2/%s %s PROPOSITION :<nom>\2 jusqu' que vous ayez donn les %d propositions",
				cs.nick, RealCmd("vote"), vote_nbprop);
		strcpy(vote[0].prop, "end");
		vote_nbprop--;
	}
	else if(!strcasecmp(cmd, "PROPOSITION"))
	{
		int i = 0;
		char *msg = NULL;

		if(parc < 2)
			return csntc(nick, "Syntaxe: %s PROPOSITION :<proposition>", RealCmd("vote"));

		if(vote_actif) return csntc(nick, "Le vote est dj actif");

		if(!vote_nbprop) return csntc(nick, "Aucun vote n'est en cours d'enregistrement");

		for(;strcmp(vote[i].prop, "end");i++);
		if(i > vote_nbprop)
			return csntc(nick, "Vous avez bien donn toutes les propositions pour le vote");

		msg = parv2msg(parc, parv, 2, MEMOLEN + 3);
		if(*msg==':') msg++;
		if(!strcmp(msg, "end"))
			return csntc(nick, "Vous ne pouvez pas donner comme proposition 'end'");

		if(strlen(msg) > MEMOLEN)
			return csntc(nick, "La longueur de la proposition ne doit pas dpasser %d caractres", MEMOLEN);

		Strncpy(vote[i].prop, msg, MEMOLEN);
		vote[i].voies = 0;
		strcpy(vote[i + 1].prop, "end");

		if(i == vote_nbprop)
		{
			vote_actif = 1;
			csntc(nick, "Le vote est maintenant \2ACTIF\2.");
			show_vote(nick);
			write_votes();
		}
		else csntc(nick, "Vous avez donn %d des %d propositions.", i + 1, vote_nbprop + 1);
	}
	else if(!strcasecmp(cmd, "CLORE"))
	{
		if(!vote_actif) return csntc(nick, "Il n'y a pas de vote actif.");

		vote_actif = 0;
		csntc(nick, "Le vote est clos, les users peuvent maintenant voir les rsultats.");
		write_votes();
	}
	else if(!strcasecmp(cmd, "RESULTS"))
	{
		int i = 0, prout[20], *sex, usr;
		float f = 0;

		if(!vote_nbprop) return csntc(nick, "Il n'y a aucun vote existant.");

		csntc(nick, "Rsultats:");

		for(;strcmp(vote[i].prop, "end");i++)
		{
			f += vote[i].voies;
			prout[i] = vote[i].voies;
		}
		sex = class_votes(prout);
		for(;i <= vote_nbprop;i++) csntc(nick, "%d. %3.2f%% [%d] \2%s", i + 1,
			(vote[sex[i]].voies / f) * 100, vote[sex[i]].voies, vote[sex[i]].prop);
		usr = CountUsers(NULL);
		csntc(nick, "%1.0f des %d users ont vots (%3.2f%%)", f, usr, (f / usr) * 100);
	}
	else csntc(nick, "La sous commande %s n'existe pas", cmd);
	return 1;
}

void write_votes(void)
{
   FILE *fp = fopen(VOTE_FILE, "w");
   int i = 0;

   if(!fp) return;

   fprintf(fp, "VOTE %d %d :%s\n", vote_actif, vote_nbprop, vote_topic);
   for(;i <= vote_nbprop;i++) fprintf(fp, "PROP %d :%s\n", vote[i].voies, vote[i].prop);
   fclose(fp);
   return;
}

void load_votes(void)
{
   FILE *fp = fopen(VOTE_FILE, "r");
   char buff[300], *field = NULL;
   int i = 0;

   if(!fp) return;
   while(fgets(buff, sizeof buff, fp))
   {
	   strip_newline(buff);
	   if((field = strchr(buff, ' '))) *field++ = 0;
	   else continue;
	   if(!strcmp(buff, "VOTE"))
	   {
		   Strncpy(vote_topic, strchr(field, ':') + 1, MEMOLEN);
		   vote_actif = atoi(strtok(field, " "));
		   vote_nbprop = atoi(strtok(NULL, " "));
	   }
	   else if(!strcmp(buff, "PROP"))
	   {
		   Strncpy(vote[i].prop, strchr(field, ':') + 1, MEMOLEN);
		   vote[i++].voies = atoi(strtok(field, " "));
	   }
   }
   strcpy(vote[i].prop, "end");
   fclose(fp);
}
#endif
